#!/bin/bash
#
# Install software while it is runnin from CD into home directory - useful in combination with persistant home
#
# GPL
#
# Author: Fabian Franz <knx-live-inst@fabian-franz.de>
#

# Constants

DESTDIR="$HOME/.dist"
export DESTDIR
TMPDIR="/tmp/knx-live-inst.$$"
MODPATH=$(pwd)
MODULES=$MODPATH/modules/
FILES="files"
INSTALL="install.sh"
mkdir -p $TMPDIR
cd $TMPDIR

function clean_exit
{
  cd ~
  rm -rf $TMPDIR
}

trap "clean_exit" EXIT

# Functions needed

function deb-unpack()
{
  ar p $1 data.tar.gz | tar xz
}

function apt-unpack()
{
  echo "Not implemented."
}

function rpm-unpack()
{
  rpm2cpio $1 | cpio --extract --make-directories
}

function tgz-unpack()
{
  tar xzf $1
}

function tbz-unpack()
{
  tar xzj $1
}

# knx-unpack filename location type md5sum
function knx-unpack()
{
  [ ! -f $1 ] && return 2
  case "$3" in
  	deb) deb-unpack $1 ;;
	apt) apt-unpack $1 ;; 
  	rpm) rpm-unpack $1 ;;
  	tgz) tgz-unpack $1 ;;
  	tbz) tbz-unpack $1 ;;
	*) 
	  echo "Unknown Package format" 
	  false
	;;
  esac
}

function setup_destdir()
{
  # setup $HOME/.bashrc
  
  mkdir -p $DESTDIR/bin
  mkdir -p $DESTDIR/lib
  mkdir -p $DESTDIR/man

  cp $HOME/.bashrc $HOME/.bashrc.templ.$$
  cat $HOME/.bashrc.templ.$$ | grep -v "DESTDIR=" | grep -v "export MANPATH=\$DESTDIR/man" | grep -v "export PATH=\$DESTDIR/bin/" | grep -v "export LD_LIBRARY_PATH=\$DESTDIR/lib" > $HOME/.bashrc
  echo "DESTDIR=$DESTDIR" >> $HOME/.bashrc
  echo "export MANPATH=\$DESTDIR/man/:\$MANPATH" >> $HOME/.bashrc
  echo "export PATH=\$DESTDIR/bin/:\$PATH" >> $HOME/.bashrc
  echo "export LD_LIBRARY_PATH=\$DESTDIR/lib:\$LD_LIBRARY_PATH" >> $HOME/.bashrc
  rm -f $HOME/.bashrc.templ.$$

}

function abort
{
  echo $1
  exit $2
}

function compare_md5sum
{
  [ "$2"="none" ] && return
  MD5SUM=$(md5sum $1 | awk '{ print $1 }')
  [ "$MD5SUM" != "$2" ] && echo "Warning: MD5DUM does not match for file $1!"
}

# Format of Files order + package_name
# So package basename location type md5sum
function get_file()
{
  [ -f $MODULES/$1/$2 ] && cp $MODULES/$1/$2 $TMPDIR
  [ -f $MODULES/$1/$2 ] || wget $3
  [ -n "$5" ] && compare_md5sum $2 $5
}

if [ "$#" -lt "1" ]; then
  echo Usage: $(basename $0) "<module-name>"
  echo "Availabe are:"
  ls $MODULES | tr ' ' \n
  exit 1
fi 

if [ ! -f "$MODULES/$1/$FILES" ];
then
  echo "Fatal: Package not found"
  echo "Availabe are:"
  ls $MODULES | tr ' ' \n
  exit 1
fi

echo "OK, Installing $1..."
setup_destdir

NUM=$(cat $MODULES/$1/files | egrep -v "^#" | wc -l | sed 's/ //g')
echo "Getting $NUM files..."
I="1"
while [ "$I" -le "$NUM" ];
do
  file=$(cat $MODULES/$1/files | egrep -v "^#" | tail +$I | head -n 1) 
  get_file $1 $file
  I=$(($I+1))
done

echo "Unpacking $NUM files..."
I="1"
while [ "$I" -le "$NUM" ];
do
  file=$(cat $MODULES/$1/files | egrep -v "^#" | tail +$I | head -n 1) 
  knx-unpack $file
  [ $? -eq 2 ] && abort "Fatal file not found!" 1
  I=$(($I+1))
done

echo "Installing program..."
sh $MODULES/$1/install.sh
echo "All done"
